@@ -1262,3 +1262,216 @@ def compute_alpha_signal_rsilo(df=None, core_tokens=['BTC', 'ETH'], rsi_lookback
                                                              plot_html=True,
                                                              write_file=True)
     return mod_data_df.copy()
+
+
+def compute_alpha_signal_ls_neutral(lo_df=None, core_tokens=['BTC', 'ETH'], extra_tokens=['AVAX'], moderating_factor = 0.1, lookback_window =90, pente_window = 30, me_center = 0.5, mix_lo = True, me_short = True,ls_signal_suffix = 'LS',aggregate_and_save = True,plot_subsignal_html = True,extract_subsignal_kpi = True):
+    import numpy as np
+    import functools
+    from cryptotoolbox.signal import signal_utility
+    from cryptotoolbox.realtime import realtime_plotting_utility
+    from cryptotoolbox.risk_metrics import riskmetrics
+    ssjs =  core_tokens + extra_tokens
+    univ = '_'.join(ssjs)
+    aggregated_signals_df = None
+    for ssj in ssjs:
+        current = f'univ{univ}_lookback_{lookback_window}_pente_{pente_window}_short_{me_short}_center_{me_center}.plkl'
+        print('#########'+current)
+
+        data_df = lo_df.copy()
+        print('computing weekly signals')
+
+        weekly_df = data_df.copy()
+        weekly_df = weekly_df[weekly_df['rebalance']]
+
+        def compute_ranked_slope(short, center, lagging_df):
+            lagging_df['rolling_slope_rank'] = lagging_df['rolling_slope'].rank(pct=True)
+            if short :
+                lagging_df['rolling_slope_rank_ls'] = 2 * lagging_df['rolling_slope_rank'] - center
+                lagging_df['rolling_slope_rank_ls'] = lagging_df['rolling_slope_rank_ls'].clip(-1,1)
+            else:
+                lagging_df['rolling_slope_rank_ls'] = lagging_df['rolling_slope_rank']
+
+            gen_sig = lagging_df['rolling_slope_rank_ls'].iloc[-1]
+            return gen_sig
+
+        def compute_slope(slope_df):
+           y = slope_df.values
+           slope = stats.linregress(np.arange(len(y)), y).slope
+           return slope
+
+        weekly_df['rolling_slope'] = weekly_df[f'close_{ssj}'].rolling(window=pente_window).apply(compute_slope)
+
+
+        go = functools.partial(compute_ranked_slope,me_short,me_center)
+        signal_df = roll(weekly_df, lookback_window).apply(go)
+
+        signal_df = signal_df.to_frame()
+        signal_df.columns = ['signal_gen']
+
+        data_df = pd.merge(data_df.copy(), signal_df.copy(),how='left', right_index=True, left_index=True)
+        data_df['signal_gen']=data_df['signal_gen'].ffill()
+
+        data_df['signal'] = data_df['signal_gen'].shift()
+        if aggregate_and_save:
+            sig_to_save = data_df[[f'close_{ssj}','signal']].copy()
+            if aggregated_signals_df is None:
+                sig_to_save.columns = [f'close_{ssj}',f'S{ls_signal_suffix}{ssj}']
+                aggregated_signals_df = sig_to_save.copy()
+            else :
+                sig_to_save.columns = [f'close_{ssj}',f'S{ls_signal_suffix}{ssj}']
+                aggregated_signals_df = pd.merge(aggregated_signals_df.copy(), sig_to_save.copy(), left_index = True, right_index=True)
+
+
+        data_df = data_df.dropna()
+        data_df['epoch_number'] = data_df['rebalance'].cumsum()
+
+        perf_df = data_df.copy()
+        perf_df=perf_df.rename(columns={f'close_{ssj}':'close'})
+        freqly_df, _ = signal_utility.reconstitute_signal_perf(data=perf_df, transaction_cost=True,
+                                                               normalization=False)
+
+        if plot_subsignal_html:
+            fig = realtime_plotting_utility.plot_multiple_time_series(freqly_df[['signal']],logy = False,drop_na_inf = False, title = ssj)
+            fig.show()
+            fig = realtime_plotting_utility.plot_multiple_time_series(freqly_df[['reconstituted_perf', 'reconstituted_under']],put_on_same_scale = False, drop_na_inf = False, title = ssj)
+            fig.show()
+            fig1 = realtime_plotting_utility.plot_multiple_time_series(freqly_df[['reconstituted_perf', 'reconstituted_under']],logy = False, put_on_same_scale = False, drop_na_inf = False, title = ssj)
+            fig1.show()
+
+
+        if extract_subsignal_kpi:
+            def compute_metrics(df, strat=None):
+                kpi_df = riskmetrics.get_kpi(df[[strat]])
+                return kpi_df.to_dict()[strat]
+
+            me_strat = 'reconstituted_perf'
+            go_comp_kpi = lambda x: compute_metrics(x, strat=me_strat)
+
+            epochkpis_df = freqly_df[['epoch_number', me_strat]].groupby(['epoch_number']).apply(go_comp_kpi)
+            epochkpis_df = epochkpis_df.sort_index()
+            epochkpis_df = pd.DataFrame.from_records(epochkpis_df.to_dict()).T
+
+
+
+            strat_data_df = freqly_df[['reconstituted_perf', 'reconstituted_under']].copy()
+            #strat_data_df = strat_data_df[strat_data_df.index >= '2021-02-01'].copy()
+            ma = market.MarketAnalyzer(strat_data_df, hourlyze = True)
+
+            print(f'computing kpis for underlying {ssj}')
+            kpi_df = ma.get_kpi().T
+            kpi_df=kpi_df.dropna()
+            kpi_df=kpi_df[~np.isnan(kpi_df.sharpe)]
+            kpi_df = kpi_df[kpi_df.index == 'reconstituted_perf']
+            print(kpi_df.head())
+
+    def curate_signals(row,under='BTC'):
+        if abs(row[f'close_{under}']) <= 1e-3:
+            return np.nan
+        else:
+            return row[f'S{ls_signal_suffix}{ssj}']
+
+    for ssj in ssjs:
+        go_curate = lambda x: curate_signals(x, under=ssj)
+        aggregated_signals_df[f'S{ls_signal_suffix}{ssj}'] = aggregated_signals_df.apply(go_curate, axis=1)
+
+#    for extra_tok in extra_tokens :
+#        aggregated_signals_df[f'S{ls_signal_suffix}{extra_tok}'] = aggregated_signals_df[f'S{ls_signal_suffix}{extra_tok}'].fillna(0.)
+
+    aggregated_signals_df = aggregated_signals_df.dropna()
+#    aggregated_signals_df = aggregated_signals_df[~aggregated_signals_df[f'S{ls_signal_suffix}BTC'].isna().values]
+#    aggregated_signals_df = aggregated_signals_df.fillna(0.)
+
+    if mix_lo:
+        aggregated_signals_df = pd.merge(aggregated_signals_df.copy(), lo_df.copy(),suffixes=['','_lo'], how='left',
+                                         left_index=True, right_index=True)
+        def compute_mix_signals(row_df, core_tokens= None, extra_tokens= None):
+            lo_expo = row_df['expo']
+            if lo_expo > 0 :
+                return {f'SF{tok}':row_df[f'SMIX{tok}'] for tok in core_tokens + extra_tokens}
+            else :
+                return {f'SF{tok}':row_df[f'SLS{tok}'] for tok in core_tokens + extra_tokens}
+
+        go_mix = lambda x : compute_mix_signals(x, core_tokens=core_tokens, extra_tokens=extra_tokens)
+        nex_signals_series = aggregated_signals_df.apply(go_mix, axis=1)
+        nex_signals_df = pd.DataFrame().from_records(nex_signals_series.values)
+        nex_signals_df.index = nex_signals_series.index
+        aggregated_signals_df = pd.merge(nex_signals_df.copy(),aggregated_signals_df.copy(),left_index=True, right_index=True)
+    else :
+        for tok in core_tokens + extra_tokens:
+            aggregated_signals_df[f'SF{tok}']=aggregated_signals_df[f'SLS{tok}']
+
+    def compute_ls_expo(row_df, universe = None, moderating_factor = np.nan):
+        bullish_ones = []
+        bearish_ones = []
+        bullish_dic = {}
+        bearish_dic = {}
+        bullish_magnitude = 0.
+        bearish_magnitude = 0.
+        compo = {}
+        for sig in [f'SF{me_tok}' for me_tok in universe]:
+            if row_df[sig] > 0. :
+                bullish_ones.append(sig)
+                bullish_dic[sig] = row_df[sig]
+                bullish_magnitude = bullish_magnitude +row_df[sig]
+            if row_df[sig]<0.:
+                bearish_ones.append(sig)
+                bearish_dic[sig] = row_df[sig]
+                bearish_magnitude = bearish_magnitude +row_df[sig]
+
+        for sig in [f'SF{me_tok}' for me_tok in universe]:
+            if row_df[sig] > 0. and bullish_magnitude>moderating_factor :
+                compo[sig] = row_df[sig]/bullish_magnitude*moderating_factor
+                continue
+            if row_df[sig] > 0.:
+                compo[sig] = row_df[sig]
+                continue
+            if row_df[sig] < 0. and abs(bearish_magnitude)>moderating_factor:
+                compo[sig] = row_df[sig]/abs(bearish_magnitude)*moderating_factor
+                continue
+            if row_df[sig] < 0.:
+                compo[sig] = row_df[sig]
+                continue
+            compo[sig] = 0.
+
+        final_long_expo = 0.
+        final_short_expo = 0.
+        for sig in [f'SF{me_tok}' for me_tok in universe]:
+            if compo[sig]>0.:
+                final_long_expo = final_long_expo + compo[sig]
+            if compo[sig]<0.:
+                final_short_expo = final_short_expo + compo[sig]
+
+        compo['long_expo']  = final_long_expo
+        compo['short_expo'] = final_short_expo
+        compo['SCASH'] = 1.-compo['long_expo']+ compo['short_expo']
+        return compo
+
+    comp_compo = lambda x : compute_ls_expo(x, universe=ssjs, moderating_factor=moderating_factor)
+    aggregated_signals_df['compo'] = aggregated_signals_df.apply(comp_compo, axis = 1)
+
+    normed_sig_df = pd.DataFrame().from_records(aggregated_signals_df['compo'].values)
+    normed_sig_df.index = aggregated_signals_df.index
+
+    for me_tok in ssjs:
+        aggregated_signals_df['return_{}'.format(me_tok)] = aggregated_signals_df['close_{}'.format(me_tok)].pct_change()
+    columns_to_keep = [f'return_{me_tok}' for me_tok in ssjs] + [f'close_{me_tok}' for me_tok in ssjs]
+    merged_df =  pd.merge(aggregated_signals_df[columns_to_keep].copy(), normed_sig_df.copy(), left_index = True, right_index = True)
+    merged_df = merged_df.iloc[1:]
+    merged_df.replace([np.inf, -np.inf], np.nan, inplace=True)
+    merged_df = merged_df.fillna(0.)
+
+    print('computing exposition')
+    def compute_strat_ret(row_df, universe = None, stable_annulalized_apy=0.03):
+        aggregated_return = 0.
+        for me_token in universe:
+            aggregated_return = aggregated_return + row_df[f'return_{me_token}'] * row_df[f'SF{me_token}']
+        daily_rate = (1+stable_annulalized_apy)**(1./365.)-1
+        aggregated_return = aggregated_return + row_df[f'SCASH']*daily_rate
+        return aggregated_return
+
+    comp_ret = lambda x : compute_strat_ret(x, universe=ssjs)
+    merged_df['STRAT_RETURN'] = merged_df.apply(comp_ret, axis=1)
+
+    merged_df['COMBO_STRAT'] = np.cumprod(1 + merged_df['STRAT_RETURN'].values)
+    merged_df['DOLLAR_COMBO_STRAT'] = 10000. * np.cumprod(1 + merged_df['STRAT_RETURN'].values)
+    return merged_df.copy()
